Feature: Database and Configuration export
  Background:
    Given A configuration exists
  #################################################
  # Configuration and Database existence irrelevant
  #################################################
  Scenario: User Exports (No Database and No Configs)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I do not want to export any databases
    And I do not want to export any configurations
    When I run the application
    Then The application should fail
    And I should see an error message containing "Nothing enabled to be exported! Exiting..."
  Scenario: User Exports (Specifies database id and disables database export)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I do not want to export any databases
    And I specify "--cge-id 1"
    When I run the application
    Then The application should fail
    And I should see an error message containing "CGE backup ID specified when database export disabled. Please verify your command."
  Scenario: User Exports (Specifies configuration directory and disables config export)
    Given I am going to run the program
    And Configurations exist at "/tmp/insxsync"
    And I am going to run an export
    And I do not want to export any configurations
    And I specify "--config-dir /tmp/insxsync"
    When I run the application
    Then The application should fail
    And I should see an error message containing "Configuration directory specified when configuration export disabled. Please verify your command."

  #################################################
  # Configuration Directory Exists
  # CGE Backup Exists
  #################################################
  Scenario: User Exports (Database and Configs)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I input the following "Password001"
    When I run the application
    Then I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see "Uploading configuration synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain configurations
    And The synchronization point should contain databases
  Scenario: User Exports (Database and Configs) (Specifies Config)
    Given I am going to run the program
    And Configurations exist at "/tmp/insxsync"
    And I am going to run an export
    And I specify "--config-dir /tmp/insxsync"
    And I input the following "Password001"
    When I run the application
    Then I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see "Uploading configuration synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain configurations
    And The synchronization point should contain databases
  Scenario: User Exports (Database and Configs) (Specifies Database)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I input the following "Password001"
    And I specify "--cge-id 1"
    When I run the application
    Then I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see "Uploading configuration synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain configurations
    And The synchronization point should contain databases
  Scenario: User Exports (Database and Configs) (Specifies Database and Config)
    Given I am going to run the program
    And Configurations exist at "/tmp/insxsync"
    And I am going to run an export
    And I specify "--config-dir /tmp/insxsync"
    And I specify "--cge-id 1"
    And I input the following "Password001"
    When I run the application
    Then I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see "Uploading configuration synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain configurations
    And The synchronization point should contain databases
  Scenario: User Exports (No Database)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I do not want to export any databases
    And I input the following "Password001"
    When I run the application
    Then I should see "Uploading configuration synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain configurations
    And The synchronization point should not contain databases
  Scenario: User Exports (No Database) (Specifies Config)
    Given I am going to run the program
    And Configurations exist at "/tmp/insxsync"
    And I am going to run an export
    And I specify "--config-dir /tmp/insxsync"
    And I do not want to export any databases
    And I input the following "Password001"
    When I run the application
    Then I should see "Uploading configuration synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain configurations
    And The synchronization point should not contain databases
  Scenario: User Exports (No Configs)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I do not want to export any configurations
    And I input the following "Password001"
    When I run the application
    Then I should see "Uploading database synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain databases
    And The synchronization point should not contain configurations
  Scenario: User Exports (No Configs) (Specifies Database)
    Given I am going to run the program
    And Configurations exist at the default location for export
    And I am going to run an export
    And I specify "--cge-id 1"
    And I do not want to export any configurations
    And I input the following "Password001"
    When I run the application
    Then I should see "Uploading database synchronization data"
    And I should see "Export successful!"
    And The synchronization export should be successful
    And The synchronization point should contain databases
    And The synchronization point should not contain configurations

  #################################################
  # Configuration Directories Do Not Exist
  # CGE Backups Exist
  #################################################
  Scenario: User Exports (Default Options)
    Given I am going to run the program
    And Configurations do not exist at the default location for export
    And I am going to run an export
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see an error message containing "Configurations not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User Exports (No Database) (Default Configuration)
    Given I am going to run the program
    And Configurations do not exist at the default location for export
    And I am going to run an export
    And I do not want to export any databases
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see an error message containing "Configurations not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User Exports (No Database) (Specifies Configuration)
    Given I am going to run the program
    And Configurations do not exist at "/tmp/insxsync"
    And I am going to run an export
    And I do not want to export any databases
    And I specify "--config-dir /tmp/insxsync"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see an error message containing "Configurations not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User exports (Default Export Actions) (Default Configuration) (Specifies CGE Backup)
    Given I am going to run the program
    And Configurations do not exist at the default location for export
    And I am going to run an export
    And I specify "--cge-id 1"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see an error message containing "Configurations not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User Exports (Default Export Actions) (Specifies Configuration) (Default CGE Backup)
    Given I am going to run the program
    And Configurations do not exist at "/tmp/insxsync"
    And I am going to run an export
    And I specify "--config-dir /tmp/insxsync"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see an error message containing "Configurations not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User exports (Default Export Actions) (Specifies Configuration) (Specifies CGE Backup)
    Given I am going to run the program
    And Configurations do not exist at "/tmp/insxsync"
    And I am going to run an export
    And I specify "--config-dir /tmp/insxsync"
    And I specify "--cge-id 1"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see "Uploading database synchronization data"
    And I should see an error message containing "Configurations not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist

  #################################################
  # Configuration Directories Exist
  # CGE Backups Do Not Exist
  #################################################
  Scenario: User Exports (No Configuration) (Specifies CGE Backup)
    Given I am going to run the program
    And I am going to run an export
    And I do not want to export any configurations
    And I specify "--cge-id 999"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see an error message containing "Specified CGE backup not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User exports (Default Export Actions) (Specifies CGE Backup) (Default Configuration)
    Given I am going to run the program
    And I am going to run an export
    And Configurations exist at the default location for export
    And I specify "--cge-id 999"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see an error message containing "Specified CGE backup not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist
  Scenario: User exports (Default Export Actions) (Specifies CGE Backup) (Specifies Configuration)
    Given I am going to run the program
    And I am going to run an export
    And Configurations exist at "/tmp/insxsync"
    And I specify "--cge-id 999"
    And I specify "--config-dir /tmp/insxsync"
    And I input the following "Password001"
    When I run the application
    Then The application should fail
    And I should see "Downloading CGE backup and extracting"
    And I should see an error message containing "Specified CGE backup not found!"
    And I should see an error message containing "Removing incomplete synchronization point"
    And The synchronization export should not exist