And(/^The database is not running$/) do
  `sc stop mysql56`
end

And(/^The database is running$/) do
  `sc start mysql56`
end

And(/^I am going to run a sync$/) do
  need_to_stop = false
  begin
    dbh = Mysql.real_connect("127.0.0.1", "root", "Password001", nil, 3306)
  rescue Mysql::Error => e
    if e.error =~ /Can't connect to MySQL server/
      step %Q{The database is running}
      need_to_stop = true
    else
      raise
    end
  ensure
    dbh.close if dbh
  end

  step %Q{I specify the command "sync"}
  dbs = `mysql -u root -p'Password001' -B -N -e 'show databases' -h 127.0.0.1 -P3306`.split("\n")
  dbs.each do |db|
    next if db == "information_schema"
    next if db == "mysql"
    next if db == "performance_schema"
    `mysql -u root -p'Password001' -B -N -e 'drop database \`#{db}\`' -h 127.0.0.1 -P3306`
  end
  @sync = true

  dir = Dir.pwd

  Dir.chdir(File.join(dir, 'features', 'support'))
  `bash db_restore.sh`

  Dir.chdir(dir)

  if need_to_stop
    step %Q{The database is not running}
  end
end

Then(/^The synchronization should be successful$/) do
  step %Q{I should see "Synchronization completed successfully."}
end

And(/^The databases should be unchanged$/) do
  dbs = `mysql -u root -p'Password001' -B -N -e 'show databases' -h 127.0.0.1 -P3306`.split("\n")
  dbs.each do |db|
    next if db == "information_schema"
    next if db == "mysql"
    next if db == "performance_schema"
    unless db =~ /(.*?)23feb/
      puts db
      raise "The databases have changed"
    end
  end
end

And(/^The databases should be updated/) do
  dbs = `mysql -u root -p'Password001' -B -N -e 'show databases' -h 127.0.0.1 -P3306`.split("\n")
  dbs.reject! do |db|
    db == "information_schema" or db == "mysql" or db == "performance_schema" or db =~ /(.*?)23feb/
  end

  if dbs.count == 0
    puts `mysql -u root -p'Password001' -B -N -e 'show databases' -h 127.0.0.1 -P3306`.split("\n")
    raise "No new databases imported."
  end
end

And(/^The config files should be updated$/) do
  raise if not File.exists?("#{@config_dir}.bak")
  raise if not File.exists?(@config_dir)
  Find.find(@config_dir) do |file|
    next if File.directory?(file)
    path = file.scan(/.*(subdomain.*)/).flatten[0]
    next if path == 'subdomain/config.ini'
    text = File.read(file)
    db_name, db_suffix = text.scan(/params.database\s*=\s*('|")([^'"]*?)(db|3[01][^'"]*|[0-2]\d[^'"]*)?('|")/).flatten
    if db_suffix == 'db'
      puts file
      puts db_name
      puts db_suffix
      raise "Config file not updated!"
    end
  end
end

And(/^The config files should be unchanged/) do
  Find.find(@config_dir) do |file|
    next if File.directory?(file)
    path = file.scan(/.*(subdomain.*)/).flatten[0]
    next if path == 'subdomain/config.ini'
    text = File.read(file).gsub("\"","'")
    db_name, db_suffix = text.scan(/params.database\s*=\s*'([^']*?)(db|3[01][^']*|[0-2]\d[^']*)?'/).flatten
    unless db_suffix == 'db'
      puts file
      puts db_name
      puts db_suffix
      raise "Config file changed!"
    end
  end
end

And(/^I specify valid database credentials$/) do
  step %Q{I specify "--user root"}
  step %Q{I specify "--password='Password001'"}
end

And(/^I specify invalid database credentials$/) do
  step %Q{I specify "--user root"}
  step %Q{I specify "--password='BADDATA'"}
end